/*
 * \brief Little helper that will abstract pathes of underlying systems
 *
 * \copyright Copyright (c) 2014-2018 Governikus GmbH & Co. KG, Germany
 */

#pragma once

#include <QCoreApplication>
#include <QStringBuilder>
#include <QLibraryInfo>

namespace governikus
{

class FileDestination
{
	private:
		FileDestination() = delete;
		~FileDestination() = delete;
		Q_DISABLE_COPY(FileDestination)

		static QString getPath()
		{
			#ifdef Q_OS_ANDROID
			return QStringLiteral("assets:");

			#elif defined(Q_OS_LINUX)
			return QStringLiteral("/usr/share/AusweisApp2");

			#else
			return QCoreApplication::applicationDirPath();

			#endif
		}


	public:
		static QString getPath(const QString& pFilename)
		{
			#if defined(Q_OS_LINUX)
			if (pFilename.compare(QStringLiteral("translations")) == 0)
				return QLibraryInfo::location(QLibraryInfo::TranslationsPath);
			else
			#endif
				return getPath() % QLatin1Char('/') % pFilename;
		}


		static QString getPath(const char* pFilename)
		{
			#if defined(Q_OS_LINUX)
			if (strcmp(pFilename, "translations") == 0)
				return QLibraryInfo::location(QLibraryInfo::TranslationsPath);
			else
			#endif
				return getPath() % QLatin1Char('/') % QString::fromUtf8(pFilename);
		}


};

} /* namespace governikus */
